package com.example.nto.config;

import com.example.nto.entity.Employee;
import com.example.nto.service.EmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.crypto.password.NoOpPasswordEncoder; // Для незашифрованных паролей
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;

import java.util.Optional;

@Configuration
@EnableWebSecurity
public class SecurityConfig extends WebSecurityConfigurerAdapter {

    @Autowired
    private EmployeeService employeeService;

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http
                .csrf().disable() // Отключ CSRF
                .authorizeRequests()
                .antMatchers("/api/auth").permitAll() // Разрешаем доступ к /auth
                .anyRequest().authenticated() // Все  запросы требуют аутентификации
                .and()
                .httpBasic(); // Включаем базу
    }

    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth.userDetailsService(userDetailsService()).passwordEncoder(NoOpPasswordEncoder.getInstance());
    }

    @Bean
    public UserDetailsService userDetailsService() {
        return username -> {
            Optional<Employee> employee = employeeService.findByLogin(username);
            if (employee.isPresent()) {
                Employee emp = employee.get();
                return org.springframework.security.core.userdetails.User.withUsername(emp.getLogin())
                        .password(emp.getPassword())
                        .roles(emp.getRole())
                        .build();
            } else {
                throw new UsernameNotFoundException("User not found with login: " + username);
            }
        };
    }
}