package com.example.nto.controller;

import com.example.nto.entity.Employee;
import com.example.nto.service.EmployeeService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;

@RestController
@RequestMapping("/api") // база
public class EmployeeController {

    private final EmployeeService employeeService;

    public EmployeeController(EmployeeService employeeService) {
        this.employeeService = employeeService;
    }

    @GetMapping("/auth") // Проверка аутентификации
    public ResponseEntity<?> authenticate(@RequestHeader("Authorization") String authHeader) {
        // Проверяем заголовок
        if (authHeader == null || !authHeader.startsWith("Basic ")) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Missing or invalid Authorization header");
        }

        // Получение логина и пароля
        String[] credentials = new String(java.util.Base64.getDecoder().decode(authHeader.substring(6))).split(":");

        String login = credentials[0];
        String password = credentials.length > 1 ? credentials[1] : "";

        Optional<Employee> employee = employeeService.findByLogin(login);

        if (employee.isPresent() && employee.get().getPassword().equals(password)) {
            return ResponseEntity.ok("Authenticated successfully");
        }

        return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid login or password");
    }

    @PreAuthorize("isAuthenticated()") // Убедитесь, что пользователь аутентифицирован
    @GetMapping("/{login}/info") // Получение информации о сотруднике
    public ResponseEntity<?> getInfo(@PathVariable String login) {
        Optional<Employee> employee = employeeService.findByLogin(login);
        if (employee.isPresent()) {
            return ResponseEntity.ok(employee.get());
        } else {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("Employee not found");
        }
    }

    @PreAuthorize("isAuthenticated()") // Убедитесь, что пользователь аутентифицирован
    @PatchMapping("/{login}/open") // Открыть дверь
    public ResponseEntity<?> openDoor(@PathVariable String login, @RequestBody Map<String, Long> payload) {
        Long code = payload.get("value");

        if (code != null) {
            Optional<Employee> employee = employeeService.findByLogin(login);
            if (!employee.isPresent()) {
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Invalid login");
            }
            if (employeeService.validateCode(login, code)) {
                return ResponseEntity.ok("Door opened");
            } else {
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Invalid code");
            }
        } else {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Invalid payload");
        }
    }

    @PreAuthorize("hasRole('admin')")
    @GetMapping("/workers")
    public ResponseEntity<List<Employee>> getAllWorkers() {
        String currentUserName = SecurityContextHolder.getContext().getAuthentication().getName();
        // Логируем информацию о текущем пользователе
        System.out.println("Current user: " + currentUserName);
        List<Employee> allEmployees = employeeService.findAll();
        return ResponseEntity.ok(allEmployees);
    }
}